var myWindowId;

// --------------------------------------------------------------------
// This is the HTML element in the popup window where we show the
// grammar checking results.
// --------------------------------------------------------------------
const granskaResultBox = document.querySelector("#granskaResults");

// ---------------------------------------------------------------
// updateOnPageChange()
// ---------------------------------------------------------------
// Initialize everything when a new page is loaded or the panel is
// opened for the first time etc.
// ---------------------------------------------------------------
function updateOnPageChange() {
    browser.tabs.executeScript({
	file: "/src/script.js"
    }).then(messageContent)
	.catch(reportError);

    var firstRun = true;
    function messageContent() {
	if(firstRun) {
	    firstRun = false;
	    if (granskaResultBox) {
		granskaResultBox.innerHTML = "";
	    } else {
		console.log("WARNING: no panel to work with?");
	    }

	    popupMap = {};
	    suggMap = {};
	    seenTexts = [];
	    
	    const gettingActiveTab = browser.tabs.query({active: true, currentWindow: true});
	    gettingActiveTab.then((tabs) => {
		browser.storage.local.set({"granskaPollFlag":false});
		browser.tabs.sendMessage(tabs[0].id, {"req":"startListening"}, messageFromPage);
		// browser.tabs.sendMessage(tabs[0].id, {"req":"sendTexts", "force":true}, textsFromPage);
	    });
	}
    }

    function reportError(error) {
	console.error(`WARNING: Could not inject content script: ${error}`);
    }
}

// --------------------------------------------------------------------
// function storageListen(changes, area)
// --------------------------------------------------------------------
// We listen to the local storage. When the injected script wants to
// send us new data, typically when the user has edited some text, it
// sets the 'granskaPollFlag' to true, and we send a message to the
// injected script with a callback function for the script to use.
// --------------------------------------------------------------------
function storageListen(changes, area) {
    browser.storage.local.get("granskaPollFlag").then(function (d) {
	if(d && d.granskaPollFlag) {
	    // Reset the flag
	    browser.storage.local.set({"granskaPollFlag":false});
	    
	    // Ask the injected script to send us the texts.
	    const gettingActiveTab = browser.tabs.query({active: true, currentWindow: true});
	    gettingActiveTab.then((tabs) => {
		browser.tabs.sendMessage(tabs[0].id, {"req":"sendTexts"}, textsFromPage);
	    });
	}			
    });
}

// --------------------------------------------------------------------
// function messageFromPage(response)
// --------------------------------------------------------------------
// No longer used
// --------------------------------------------------------------------
function messageFromPage(response) {
    // console.log("got message from page, ", response);
}

// --------------------------------------------------------------------
// Our copy of texts we have seen so far. Each element in the list
// contains the text content ('text'), a flag indicating if the text
// is in Swedish or not ('se'), the grammar checking results
// ('granska'), and the HTML to display for the grammar checking
// results ('html').
// --------------------------------------------------------------------
var seenTexts = [];

// --------------------------------------------------------------------
// function textsFromPage(response)
// --------------------------------------------------------------------
// Callback function for the injected script to use to send us the
// texts it has found.
// --------------------------------------------------------------------
function textsFromPage(response) {
    if(response) {
	var needToUpdate = false;

	popupMap = response.pm;
	suggMap = response.sm;
	
	for(var i = 0; i < response.st.length; i++) {

	    oldT = {"text":"", "se":false, "granska":[], "html":""};
	    if(i < seenTexts.length) {
		var oldT = seenTexts[i];
	    }
	    var newT = response.st[i];

	    if(oldT.text != newT.text) {
		// if this text has been modified...
		if(newT.se || (oldT.se && oldT.html != "")) {
		    needToUpdate = true;
		}
	    } else if(oldT.html != newT.html) {
		needToUpdate = true;
	    }
	}

	for(var i = response.st.length; i < seenTexts.length; i++) {
	    if(seenTexts[i].html != "") {
		needToUpdate = true;
	    }
	}
	
	seenTexts = response.st;
	
	if(needToUpdate) {
	    updateHTML();
	}
    }
}

// ---------------------------------------------------------------------------------
// function updateHTML()
// ---------------------------------------------------------------------------------
// Update the HTML in the popup window by basically concatenating the
// HTML for the grammar checking results for each text.
// ---------------------------------------------------------------------------------
function updateHTML() {
    var res = "";
    
    for(var i = 0; i < seenTexts.length; i++) {
	if(seenTexts[i].se && seenTexts[i].html != "") {
	    if(res != "") {
		res += "<hr>";
	    }
	    res += seenTexts[i].html;
	}
    }

    if (granskaResultBox) {
	granskaResultBox.innerHTML = res;

	// Go through the HTML and add functions to the elements that
	// should be clickable
	for(var p in popupMap) {
	    if(popupMap.hasOwnProperty(p)) {
		// for all text segments with some grammar checking
		// annotation, show the hidden info when the segment
		// is clicked
		var element = document.getElementById(p);
		if(element) {
		    element.addEventListener("click", function(event) {
			var source = event.target || event.srcElement;
			
			if(popupMap.hasOwnProperty(source.id)) {
			    var arg = popupMap[source.id];
			    var popup = document.getElementById(arg);
			    if(popup) {
				popup.classList.toggle("show");
			    }
			}
		    });

		    var arg = popupMap[p];
		    var pup = document.getElementById(arg);
		    if(pup) {
			pup.addEventListener("click", function(event) {
			    var source = event.target || event.srcElement;
			    if(source) {
				if(source.className == "granskaPopupText show") {
				    source.classList.toggle("show");
				} else if(source.className == "granskaPopupText-content") {
				    source.parentElement.classList.toggle("show");
				}
			    }
			});
		    }
		}
	    }
	}

	for(var p in suggMap) {
	    if(suggMap.hasOwnProperty(p)) {
		// For all suggested corrections from the grammar
		// checker, update the original text when the
		// suggestion is clicked.
		var element = document.getElementById(p);
		if(element) {
		    element.addEventListener("click", function(event) {
			var source = event.target || event.srcElement;
			
			if(suggMap.hasOwnProperty(source.id)) {
			    var arg = suggMap[source.id];

	    		    var s = arg[0]; // sentence no
	    		    var e = arg[1]; // error no
    	    		    var elemIdx = arg[2]; // seenTexts[elemIdx]
	    		    var sg = arg[3]; // suggestion text

			    if(elemIdx < seenTexts.length) {
				if(seenTexts[elemIdx].granska) {
    	    			    var sentences = seenTexts[elemIdx].granska;
				    
    	    			    if (sentences !== undefined && s < sentences.length) {
    	    				if (sentences[s].length > 2 && sentences[s][2].length > e) {
    	    				    pushChangeToElement(sentences, s, e, sg, elemIdx);
					}
				    }
				}
			    }
			}
		    });
		}
	    }
	}
    } else {
	console.log("WARNING: could not find the HTML element for Granska annotations.");
    }

    return res;
}

// ---------------------------------------------------------------------------------
// function pushChangeToElement(granskaSentences, sIdx, eIdx, chosenSuggestion, elemIdx)
// ---------------------------------------------------------------------------------
// When a suggested correction from the grammar checker is clicked,
// ask the injected script to update the original text in accordance
// with the suggestion.
// ---------------------------------------------------------------------------------
function pushChangeToElement(granskaSentences, sIdx, eIdx, chosenSuggestion, elemIdx) {
    const gettingActiveTab = browser.tabs.query({active: true, currentWindow: true});
    gettingActiveTab.then((tabs) => {
	browser.tabs.sendMessage(tabs[0].id, {"req":"pushCorrectionToPage", "granskaSentences":granskaSentences, "sentIdx":sIdx, "errIdx":eIdx, "chosenSuggestion":chosenSuggestion, "elIdx":elemIdx}, messageFromPage);
	browser.tabs.sendMessage(tabs[0].id, {"req":"sendTexts"}, textsFromPage);
    });
    
    return "";
}

// ----------------------------------------------------------------------------------
// popupMap stores information on what grammar checking annotation
// HTML element should get a callback function and what that function
// should do.
// ----------------------------------------------------------------------------------
// suggMap information on what grammar checking suggestion HTML
// element should get a callback function and what that function
// should do.
// ----------------------------------------------------------------------------------
var popupMap = {};
var suggMap = {};

/*
  Update content when a new tab becomes active.
*/
if(browser.tabs.onActivated.hasListener(updateOnPageChange)) {
    browser.tabs.onActivated.removeListener(updateOnPageChange);
}
browser.tabs.onActivated.addListener(updateOnPageChange);

/*
  Update content when a new page is loaded into a tab.
*/
if(browser.tabs.onUpdated.hasListener(updateOnPageChange)) {
    browser.tabs.onUpdated.removeListener(updateOnPageChange);
}
browser.tabs.onUpdated.addListener(updateOnPageChange);

/*
  When the sidebar loads, get the ID of its window,
  and update its content.
*/
browser.windows.getCurrent({populate: true}).then((windowInfo) => {
    myWindowId = windowInfo.id;
    updateOnPageChange();
});

if(browser.storage.onChanged.hasListener(storageListen)) {
    browser.storage.onChanged.removeListener(storageListen);
}
browser.storage.onChanged.addListener(storageListen);
